"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const chai = require("chai");
const LazyFormatLogger = require("lazy-format-logger");
const lazyboyjs_1 = require("../src/lazyboyjs");
let expect = chai.expect;
describe('LazyBoyAsync', () => {
    let testInstanceId = "";
    let dropDbsAfterTest = true;
    describe('Default error', () => {
        it('Should log a line', () => {
            new lazyboyjs_1.lazyboyjs.LazyBoyError("Test logging");
        });
        it('Should NOT log a line', () => {
            lazyboyjs_1.lazyboyjs.setLevel(LazyFormatLogger.LogLevel.CRITICAL);
            new lazyboyjs_1.lazyboyjs.LazyBoyError("Test logging");
        });
    });
    describe('Default options', () => {
        it('If no options are passed in then default values should be applied', () => {
            lazyboyjs_1.lazyboyjs.setLevel(LazyFormatLogger.LogLevel.VERBOSE);
            var l = new lazyboyjs_1.lazyboyjs.LazyBoyAsync();
            expect(l.options.host).equal("127.0.0.1");
            expect(l.options.port).equal(5984);
            expect(l.options.prefix).equal("lazy");
            expect(l.options.autoConnect).equal(true);
            expect(Object.keys(l.options.views).length).equal(0);
        });
    });
    describe('AutoConnect false', () => {
        it('Should not connect if autoConnect is set to false', () => {
            var l = new lazyboyjs_1.lazyboyjs.LazyBoyAsync({ autoConnect: false });
            expect(l.hasConnection()).equal(false);
        });
    });
    describe('AutoConnect true', () => {
        it('Should try connect if no configuration is passed in', () => {
            var l = new lazyboyjs_1.lazyboyjs.LazyBoyAsync();
            expect(l.hasConnection()).to.equal(true);
        });
    });
    describe('Create database', () => {
        it("Should create a database with the name 'lazy_test'", () => __awaiter(this, void 0, void 0, function* () {
            var l = new lazyboyjs_1.lazyboyjs.LazyBoyAsync();
            let report = yield l.InitializeDatabaseAsync('test');
            expect(report.name).to.equal('lazy_test');
            expect(report.status).to.equal(lazyboyjs_1.lazyboyjs.DbCreateStatus.Created_Without_Views);
        }));
    });
    describe('Create multiple databases', () => {
        it("Should create databases with the name " +
            "'lazy_test_multiple1','lazy_test_multiple2','lazy_test_multiple3'", () => __awaiter(this, void 0, void 0, function* () {
            var l = new lazyboyjs_1.lazyboyjs.LazyBoyAsync().Databases('test_multiple1', 'test_multiple2', 'test_multiple3');
            let report = yield l.InitializeAllDatabasesAsync();
            expect(report.success.length).to.equal(3);
        }));
    });
    describe('Create database with views', () => {
        it("Should create a database with name 'lazy_views' and add Views", () => __awaiter(this, void 0, void 0, function* () {
            function emit(...and) {
            }
            let LazyOptions = {
                autoConnect: true,
                views: {
                    "lazy_views": {
                        version: 1,
                        type: "javascript",
                        views: {
                            "fromNameToId": {
                                map: function (doc) {
                                    if (doc.hasOwnProperty("instance") && doc.instance.hasOwnProperty("name")) {
                                        emit(doc.instance.name, doc._id);
                                    }
                                },
                                reduce: "_count()"
                            },
                            "fromNameToIdReduce": {
                                map: function (doc) {
                                    if (doc.hasOwnProperty("instance") && doc.instance.hasOwnProperty("name")) {
                                        emit(doc.instance.name, doc._id);
                                    }
                                },
                                reduce: "_count()"
                            }
                        }
                    }
                }
            };
            var LazyBoy = new lazyboyjs_1.lazyboyjs.LazyBoyAsync(LazyOptions).Databases('views');
            let report = yield LazyBoy.InitializeAllDatabasesAsync();
            expect(report.success[0].name).to.equal("lazy_views");
            expect(report.success[0].status).to.equal(lazyboyjs_1.lazyboyjs.DbCreateStatus.Created);
        }));
        it('Should add view to database to existing LazyDesignView', () => __awaiter(this, void 0, void 0, function* () {
            function emit(...and) {
            }
            var myNewView = {
                map: function (doc) {
                    if (doc.hasOwnProperty("instance") && doc.instance.hasOwnProperty("name")) {
                        emit(doc.instance.name, doc._id);
                    }
                },
                reduce: "_count()"
            };
            var LazyBoy = new lazyboyjs_1.lazyboyjs.LazyBoyAsync().Databases('views');
            LazyBoy.ConnectAsync();
            let report = yield LazyBoy.AddViewAsync('views', 'myNewView', myNewView);
            expect(report.error).to.equal(null);
            expect(report.result).to.equal(true);
        }));
    });
    describe('Create a instance', () => {
        it("Should add an instance in the database named 'lazy_views'", () => __awaiter(this, void 0, void 0, function* () {
            var l = new lazyboyjs_1.lazyboyjs.LazyBoyAsync().Databases('views');
            yield l.ConnectAsync();
            let report = yield l.AddEntryAsync('views', { data: { name: 'TheInstance', otherValue: 'test' } });
            expect(report.error).to.equal(null);
            expect(report.result).to.equal(lazyboyjs_1.lazyboyjs.InstanceCreateStatus.Created);
            expect(report.entry._id).to.not.equal(null);
            testInstanceId = report.entry._id;
        }));
    });
    describe('Get result of view', () => {
        it("Should return the id of an instance in the database 'lazy_views'", () => __awaiter(this, void 0, void 0, function* () {
            var l = new lazyboyjs_1.lazyboyjs.LazyBoyAsync().Databases('views');
            yield l.ConnectAsync();
            let report = yield l.GetViewResultAsync('views', 'fromNameToId', { key: 'TheInstance', reduce: false });
            expect(report.error).to.equal(null);
            expect(report.result.length > 0).to.equal(true);
            expect(report.result[0].id).to.equal(testInstanceId);
        }));
        it("Should return the id of an instance in the database 'lazy_views' not reduce", () => __awaiter(this, void 0, void 0, function* () {
            var l = new lazyboyjs_1.lazyboyjs.LazyBoyAsync().Databases('views');
            yield l.ConnectAsync();
            let report = yield l.GetViewResultAsync('views', 'fromNameToIdReduce', {
                key: 'TheInstance',
                reduce: false
            });
            expect(report.error).to.equal(null);
            expect(report.result.length > 0).to.equal(true);
            expect(report.result[0].id).to.equal(testInstanceId);
        }));
    });
    describe('Get entry from database', () => {
        it('Should return a complete entry', () => __awaiter(this, void 0, void 0, function* () {
            var l = new lazyboyjs_1.lazyboyjs.LazyBoyAsync().Databases('views');
            yield l.ConnectAsync();
            let report = yield l.GetEntryAsync('views', testInstanceId);
            expect(report.error).to.equal(null);
            expect(report.data._id).to.equal(testInstanceId);
            expect(report.data.instance.name).to.equal('TheInstance');
        }));
    });
    if (dropDbsAfterTest) {
        describe('Drop one database', () => {
            it("Should drop the database named 'lazy_test'", () => __awaiter(this, void 0, void 0, function* () {
                var l = new lazyboyjs_1.lazyboyjs.LazyBoyAsync().Databases('test');
                yield l.ConnectAsync();
                let report = yield l.DropDatabaseAsync('test');
                expect(report.error).to.equal(null);
                expect(report.status).to.equal(lazyboyjs_1.lazyboyjs.DbDropStatus.Dropped);
            }));
        });
        describe('Drop multiple databases', () => {
            it("Should drop all the databases of this test", () => __awaiter(this, void 0, void 0, function* () {
                var l = new lazyboyjs_1.lazyboyjs.LazyBoyAsync().Databases('views', 'test_multiple1', 'test_multiple2', 'test_multiple3');
                yield l.ConnectAsync();
                let report = yield l.DropAllDatabasesAsync();
                expect(report.success.length).to.equal(4);
                expect(report.fail.length).to.equal(0);
            }));
        });
    }
});
